local addonName, addonTable = ...

AmendUIFuncs = {}
addonTable.Funcs = AmendUIFuncs

local au = AmendUIFuncs

au.localizedClass, au.englishClass = UnitClass("player")

au.DEBUG = false

au.loader = CreateFrame("FRAME")


-- Used to create the global options check buttons in the Interface Options Menu
au.GlobalPrefData = {}
au.GlobalPrefData[1] = {["name"] = "MainMenuBar", ["cbtext"] = "Enable Custom Main Menu Bar"}
au.GlobalPrefData[2] = {["name"] = "Gryphons", ["cbtext"] = "Remove Main Menu Bar Gryphons"}
au.GlobalPrefData[3] = {["name"] = "LandingButton", ["cbtext"] = "Reposition 'Request Landing' Button "}
au.GlobalPrefData[4] = {["name"] = "Minimap", ["cbtext"] = "Enable Minimap Tweaks & Scrollwheel Zoom"}


au.global_prefs = {}

au.default_g_prefs = {}
for i=1,#au.GlobalPrefData do
  local prefdata = au.GlobalPrefData[i]
  au.default_g_prefs[prefdata.name] = true
end

au.char_prefs = {}

au.default_c_prefs = {
  ["aui_stancebar"] = false,
  ["aui_petbar"] = true,
  ["aui_petbarposition"] = 3,
  ["aui_petxp"] = true,
  ["aui_petxpdisplay"] = 1,
  ["aui_druidbar"] = true
}

au.stance_bar_classes = {
  "DRUID",
  "WARRIOR",
  "ROGUE"
}

-- Copy over the default pref settings for new users, and eliminate 
-- outdated settings for existing users after future addon updates
function au.copyDefaultPrefs(src, dst)
    if not src then return { } end
    if not dst then dst = { } end
    --print("- Removing old variables from dst -")
    for vname, vvalue in pairs(dst) do
      --print(vname)
      if type(vvalue) ~= type(src[vname]) then
        --print("Removed")
        dst[vname] = nil
      end
    end
   --print("- Adding new variables to dst -")
    for vname, vvalue in pairs(src) do
      --print(vname)
      if type(vvalue) ~= type(dst[vname]) then
          --print("Added")
          dst[vname] = vvalue
      end
    end
    return dst
end

-- Custom function to determine table.len
function au.getTableSize(t)
  if(type(t) ~= "table") then
    return 0
  end
  local count = 0
  for _, __ in pairs(t) do
      count = count + 1
  end
  return count
end

-- Addon loading events
function au.loader.OnEvent(self, event, ...)
    --print(...)
    --if(event == "PLAYER_LOGIN" and addonName == ...) then
      --No current PLAYER_LOGIN event functions
    --elseif(event == "ADDON_LOADED" and addonName == ...) then
    --print(...)
    if(event == "ADDON_LOADED" and addonName == ...) then
      AmendUIGlobalPrefs = au.copyDefaultPrefs(au.default_g_prefs,AmendUIGlobalPrefs)
      au.global_prefs = AmendUIGlobalPrefs

      AmendUICharPrefs = au.copyDefaultPrefs(au.default_c_prefs,AmendUICharPrefs)
      au.char_prefs = AmendUICharPrefs

      --Class_Functions.lua
      ----Class_Druid.lua
      ----Class_Hunter.lua
      au.SetupClassSpecificFunctions()

      --MainMenuBar.lua
      au.SetupMainMenuBar()
      au.SetupGoldHUD()

      --MiniMap.lua
      au.SetupMinimap()

      if(au.DEBUG == true) then
          print("Global Vars:")
          for vname, vvalue in pairs(au.global_prefs ) do
              print(vname, vvalue)
          end
          print("Character Specific Vars:")
          for vname, vvalue in pairs(au.char_prefs ) do
              print(vname, vvalue)
          end
      end
    end
end

--au.loader:RegisterEvent("PLAYER_LOGIN")
--au.loader:RegisterEvent("PLAYER_ENTERING_WORLD")
au.loader:RegisterEvent("ADDON_LOADED")
au.loader:SetScript("OnEvent", au.loader.OnEvent)

--Slash command shortcuts to the Interface Options Menu
SLASH_AMENDMENTSUI1 = "/amendui"
SLASH_AMENDMENTSUI2 = "/amendmentsui"
SLASH_AMENDMENTSUI3 = "/amendmentstotheui"
SlashCmdList["AMENDMENTSUI"] = function(msg)
    InterfaceOptionsFrame_OpenToCategory("AmendmentsUI")
    InterfaceOptionsFrame_OpenToCategory("AmendmentsUI")
end


--Toggled on when an options change is made that requires a UI reload
au.ui_needs_reload = false

--Functions added to this table to be run after options are set that don't require a UI reload
au.ui_refresh_funcs = {}

--Interface Options Addons Panel 
au.Options = {}
au.Options.panel = CreateFrame( "Frame", "AmendmentsUI", UIParent )
-- Set the name for the Category for the Options Panel
au.Options.panel.name = "AmendmentsUI"
-- Add the panel to the Interface Options
InterfaceOptions_AddCategory(au.Options.panel)

--[[
-- Make a child panel
au.Options.childpanel = CreateFrame( "Frame", "AmendmentsUIChild", au.Options.panel)
au.Options.childpanel.name = "MyChild"
-- Specify childness of this panel (this puts it under the little red [+], instead of giving it a normal AddOn category)
au.Options.childpanel.parent = au.Options.panel.name
-- Add the child to the Interface Options
InterfaceOptions_AddCategory(au.Options.childpanel)
]]

au.Options.panel.global_pos_y = 0
au.Options.panel.char_pos_y = -180

au.Options.options_maintitle_text = "Amendments to the UI"
au.Options.options_maintitle = au.Options.panel:CreateFontString("Title","ARTWORK")
au.Options.options_maintitle:SetFontObject(GameFontNormalLarge)
au.Options.options_maintitle:SetJustifyH("LEFT")
au.Options.options_maintitle:SetJustifyV("TOP")
au.Options.options_maintitle:SetPoint("TOPLEFT", 16, au.Options.panel.global_pos_y-16)
au.Options.options_maintitle:SetText("|cff00aaff"..au.Options.options_maintitle_text.."|r")

--Var used to determine which saved var a button reads and changes.
--Must exactly match the saved var name in default_c_prefs/default_g_prefs
au.Options.panel.button_var_name = nil


--Function to setup a newly created drop down menu for the Options panel
function au.Options.SetupDropDownMenu(dropDownFrame,ddtext,ddwidth)
  dropDownFrame.dropdown = true
  dropDownFrame.text = dropDownFrame:CreateFontString(nil, 'ARTWORK', "GameFontNormalSmall")
  dropDownFrame.text:SetPoint("BOTTOMLEFT", dropDownFrame, "TOPLEFT", 13, 0)
  dropDownFrame.text:SetJustifyH("LEFT")
  dropDownFrame.text:SetText(ddtext)

  -- Create and bind the initialization function to the drop down menu
  UIDropDownMenu_Initialize(dropDownFrame, function(self, level, menuList)
    for k,v in pairs(dropDownFrame.itemlist) do
      local info = UIDropDownMenu_CreateInfo()
      info.text = v
      info.value = v
      info.owner = dropDownFrame
      info.func = AmendUIOptionsDropDownClicked
      info.justifyH = "LEFT"
      UIDropDownMenu_AddButton(info)
    end
  end)

  UIDropDownMenu_SetWidth(dropDownFrame, ddwidth)
  UIDropDownMenu_SetButtonWidth(dropDownFrame, ddwidth*2)
  UIDropDownMenu_SetSelectedID(dropDownFrame,1)
end





-- *************** Global Options ********************

local cbpos = au.Options.panel.global_pos_y-50
for i=1,#au.GlobalPrefData do
  local prefdata = au.GlobalPrefData[i]
  local cbframe = CreateFrame("CheckButton","AmendUIPrefs"..prefdata.name,au.Options.panel,"AmendmentsUICheckButtonTemplate")
  cbframe:SetPoint("TOPLEFT",40,cbpos)
  cbframe.text:SetText(prefdata.cbtext)
  cbframe.button_var_name = prefdata.name
  cbpos = cbpos-30
end

-- ***********************************

-- *************** Character Specific Options ********************
--Will only setup character specific options for classes in this table
au.Options.char_options_classes = {
  "DRUID",
  "WARRIOR",
  "ROGUE",
  "HUNTER",
  "WARLOCK"
  --"MAGE",
  --"PRIEST",
  --"SHAMAN",
  --"PALADIN"
}

if(tContains(au.Options.char_options_classes, au.englishClass)) then
  au.Options.options_chartitle_text = "Character Specific Options"
  au.Options.options_chartitle = au.Options.panel:CreateFontString("Title","ARTWORK")
  au.Options.options_chartitle:SetFontObject(GameFontNormal)
  au.Options.options_chartitle:SetJustifyH("LEFT")
  au.Options.options_chartitle:SetJustifyV("TOP")
  au.Options.options_chartitle:SetPoint("TOPLEFT", 16, au.Options.panel.char_pos_y)
  au.Options.options_chartitle:SetText("|cffb3e6ff"..au.Options.options_chartitle_text.."|r")
end


--Option to remove the stance bar
if(tContains(au.stance_bar_classes, au.englishClass)) then
  local opcb_stancebar = CreateFrame("CheckButton","AmendUIPrefsStanceBar",au.Options.panel,"AmendmentsUICheckButtonTemplate")
  opcb_stancebar:SetPoint("TOPLEFT",40,au.Options.panel.char_pos_y-20)
  if(au.englishClass == "DRUID") then
    opcb_stancebar.text:SetText("(Druid) Remove Shapeshifting Bar")
  elseif(au.englishClass == "ROGUE") then
    opcb_stancebar.text:SetText("(Rogue) Remove Fixed Stealth Button")
  else
    opcb_stancebar.text:SetText("(Warrior) Remove Stance Bar")
  end
  opcb_stancebar.button_var_name = "aui_stancebar"
end

au.Options.pet_bar_classes = {
  "HUNTER",
  "WARLOCK"
}

--Option to tweak and move the pet bar for Hunters and Warlocks
if(tContains(au.Options.pet_bar_classes, au.englishClass)) then
  local opcb_petbar = CreateFrame("CheckButton","AmendUIPrefsPetBar",au.Options.panel,"AmendmentsUICheckButtonTemplate")
  opcb_petbar:SetPoint("TOPLEFT",40,au.Options.panel.char_pos_y-20)
  if(au.englishClass == "HUNTER") then
    opcb_petbar.text:SetText("(Hunter) Use Adjusted Pet Action Bar")
  else
    opcb_petbar.text:SetText("(Warlock) Use Adjusted Pet Action Bar")
  end
  opcb_petbar.button_var_name = "aui_petbar"

  local PetBarPositionDD = CreateFrame("Frame", "AmendUIPrefsPetBarPositionDD", au.Options.panel, "UIDropDownMenuTemplate")
  PetBarPositionDD:SetPoint("LEFT",opcb_petbar,"RIGHT",35,-40)

  PetBarPositionDD.itemlist = {}
  PetBarPositionDD.itemlist[1] = "On Left Action Bar"
  PetBarPositionDD.itemlist[2] = "Above Left Action Bar"
  PetBarPositionDD.itemlist[3] = "On Right Action Bar"
  PetBarPositionDD.itemlist[4] = "Above Right Action Bar"
  PetBarPositionDD.button_var_name = "aui_petbarposition"

  au.Options.SetupDropDownMenu(PetBarPositionDD,"Pet Action Bar Position:",130)

  PetBarPositionDD.RefreshFunction = function() au.PositionPetBar(1) end

  opcb_petbar.dependent_buttons = {}
  opcb_petbar.dependent_buttons[1] = PetBarPositionDD

end

--Option to show the druid bear/cat mana bar
if(au.englishClass == "DRUID") then
  local opcb_druidbar = CreateFrame("CheckButton","AmendUIPrefsDruidBar",au.Options.panel,"AmendmentsUICheckButtonTemplate")
  opcb_druidbar:SetPoint("TOPLEFT",40,au.Options.panel.char_pos_y-50)
  opcb_druidbar.text:SetText("Enable Druid Bear/Cat Mana Bar")
  opcb_druidbar.button_var_name = "aui_druidbar"
end

--Hunter pet xp bar optionss  
if(au.englishClass == "HUNTER") then
  local opcb_petxp = CreateFrame("CheckButton","AmendUIPrefsPetXP",au.Options.panel,"AmendmentsUICheckButtonTemplate")
  opcb_petxp:SetPoint("TOPLEFT",40,au.Options.panel.char_pos_y-90)
  opcb_petxp.text:SetText("(Hunter) Enable Pet XP Bar")
  opcb_petxp.button_var_name = "aui_petxp"

  local PetXPBarDD = CreateFrame("Frame", "AmendUIPrefsPetXPBarDD", au.Options.panel, "UIDropDownMenuTemplate")
  PetXPBarDD:SetPoint("LEFT",opcb_petxp,"RIGHT",35,-40)

  PetXPBarDD.itemlist = {}
  PetXPBarDD.itemlist[1] = "Always"
  PetXPBarDD.itemlist[2] = "Outside of Combat"
  --PetXPBarDD.itemlist[3] = "When Targeting Pet"
  PetXPBarDD.button_var_name = "aui_petxpdisplay"

  au.Options.SetupDropDownMenu(PetXPBarDD,"Show Pet XP Bar:",130)

  PetXPBarDD.RefreshFunction = function() au.SetPetXPBarDisplayType(1) end

  opcb_petxp.dependent_buttons = {}
  opcb_petxp.dependent_buttons[1] = PetXPBarDD

end


-- ***********************************

--Stores the changed options to be saved when the Okay button is clicked
au.Options.pref_changes = {}

--Called when opening the menu - sets options buttons to the current saved state
au.Options.panel.refresh = function()
    _G["AmendUIPrefsMainMenuBar"]:SetChecked(au.global_prefs.MainMenuBar)
    _G["AmendUIPrefsGryphons"]:SetChecked(au.global_prefs.Gryphons)
    _G["AmendUIPrefsLandingButton"]:SetChecked(au.global_prefs.LandingButton)
    _G["AmendUIPrefsMinimap"]:SetChecked(au.global_prefs.Minimap)
    if(tContains(au.stance_bar_classes, au.englishClass)) then
      _G["AmendUIPrefsStanceBar"]:SetChecked(au.char_prefs.aui_stancebar)
    end
    if(tContains(au.Options.pet_bar_classes, au.englishClass)) then
      _G["AmendUIPrefsPetBar"]:SetChecked(au.char_prefs.aui_petbar)

      local dropDown = _G["AmendUIPrefsPetBarPositionDD"]
      local dd_num = au.char_prefs.aui_petbarposition
      UIDropDownMenu_SetSelectedID(dropDown,dd_num)
      UIDropDownMenu_SetText(dropDown,dropDown.itemlist[dd_num])

      au.EnableDisable(dropDown,au.char_prefs.aui_petbar)

    end
    if(au.englishClass == "HUNTER") then
      _G["AmendUIPrefsPetXP"]:SetChecked(au.char_prefs.aui_petxp)

      local dropDown = _G["AmendUIPrefsPetXPBarDD"]
      local dd_num = au.char_prefs.aui_petxpdisplay
      UIDropDownMenu_SetSelectedID(dropDown,dd_num)
      UIDropDownMenu_SetText(dropDown,dropDown.itemlist[dd_num])

      au.EnableDisable(dropDown,au.char_prefs.aui_petxp)
    end
    if(au.englishClass == "DRUID") then
      _G["AmendUIPrefsDruidBar"]:SetChecked(au.char_prefs.aui_druidbar)
    end
    au.Options.pref_changes = {}
end


function AmendUIOptionsCheckButtonClicked(self)
    local state = self:GetChecked()
    au.Options.pref_changes[self.button_var_name] = state

    if(au.getTableSize(self.dependent_buttons) >= 1) then
      for i=1,#self.dependent_buttons do
        au.EnableDisable(self.dependent_buttons[i],state)
      end
    end

    if(self.RefreshFunction ~= nil) then
      au.ui_refresh_funcs[self.button_var_name] = self.RefreshFunction
    else
      au.ui_needs_reload = true
    end
end

function AmendUIOptionsDropDownClicked(self)
  UIDropDownMenu_SetSelectedID(self.owner,self:GetID())
  au.Options.pref_changes[self.owner.button_var_name] = self:GetID()

  if(self.owner.RefreshFunction ~= nil) then
    au.ui_refresh_funcs[self.owner.button_var_name] = self.owner.RefreshFunction
  else
    au.ui_needs_reload = true
  end
end

function au.EnableDisable(button,state)
  if(button.dropdown == true) then
    if(state == true) then
      UIDropDownMenu_EnableDropDown(button)
    else
      UIDropDownMenu_DisableDropDown(button)
    end
  else
    button:SetEnabled(state)
  end
  if(state == true) then
    if(button.dropdown == true) then
      button.text:SetTextColor(1,0.82,0,1)
    else
      button.text:SetTextColor(1,1,1,1)
    end
  else
    button.text:SetTextColor(0.5,0.5,0.5,1)
  end
end

--Dialogue box to confirm reloading the UI
StaticPopupDialogs["AmendUI Options Reload UI"] = {
    text = "The UI needs to be reloaded for the changes to 'Amendments to the UI' to take effect. Reload now?",
    button1 = "Yes",
    button2 = "No",
    OnAccept = function()
        ReloadUI()
    end,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    preferredIndex = 1,
  }

--Options Panel Okay Button
--Saves any changes made and reloads/refreshes the UI if needed
function au.Options.panel.okay()
    xpcall(function()
        --print("Panel Okay Clicked")
        local ui_refresh = au.getTableSize(au.ui_refresh_funcs)
        if(au.ui_needs_reload or ui_refresh >= 1) then
            for vname, vvalue in pairs(au.Options.pref_changes) do
                --print(vname, vvalue)
                --if(tContains(au.global_prefs, vname)) then
                if(type(vvalue) == type(au.global_prefs[vname])) then
                  --print("Global Value")
                  au.global_prefs[vname] = vvalue
                else
                  --print("Char Value")
                  au.char_prefs[vname] = vvalue
                end
            end
            if(ui_refresh) then
              for vn, vf in pairs(au.ui_refresh_funcs) do
                vf()
              end
            end
            au.ui_refresh_funcs = {}
            if(au.ui_needs_reload) then
              StaticPopup_Show ("AmendUI Options Reload UI")
            end
        end
    end, geterrorhandler())
end


