local addonName, addonTable = ...
local au = AmendUIFuncs

function au.SetupClassSpecificFunctions()
    --localizedClass, englishClass = UnitClass("player")
    if(tContains(au.stance_bar_classes, au.englishClass)) then
        if(au.char_prefs.aui_stancebar == true) then
            RegisterStateDriver(StanceButton1, "visibility", "hide")
            RegisterStateDriver(StanceBarFrame, "visibility", "hide")
            end
        if(au.englishClass == "DRUID") then
            if(au.char_prefs.aui_druidbar == false) then return end
            au.SetupDruidManaBar()
        end

    elseif(au.englishClass == "HUNTER" or au.englishClass == "WARLOCK") then
        if(au.char_prefs.aui_petxp == true and au.englishClass == "HUNTER") then
            au.SetupHunterPetXPBar()
            au.PetActiveCheck()
        end
        au.PetBarSetup()
    end
end


function au.PetBarSetup()
    if(au.char_prefs.aui_petbar == false) then return end

    PetActionBarFrame:SetFrameStrata("BACKGROUND")

    au.PositionPetBar()

    SlidingActionBarTexture0:SetAlpha(0)
    SlidingActionBarTexture1:SetAlpha(0)

    for i=1,10 do
        _G["PetActionButton"..i]:ClearAllPoints()
        _G["PetActionButton"..i]:SetParent(au.PetBarFrame)
    end

    PetActionButton1:SetPoint("BOTTOMLEFT",au.PetBarFrame)
    PetActionButton4:SetPoint("LEFT",PetActionButton1,"RIGHT",8,0)

    for i=5,10 do
        _G["PetActionButton"..i]:SetPoint("LEFT",_G["PetActionButton"..i-1],"RIGHT",6,0)
    end

    au.AmendUIPetFollowStopFrame = CreateFrame("Frame", "AmendUIPetFollowStopFrame", au.PetBarFrame, "SecureHandlerStateTemplate")
    au.AmendUIPetFollowStopFrame:SetPoint("BOTTOMLEFT",_G["PetActionButton9"],"TOPLEFT",0,4);
    au.AmendUIPetFollowStopFrame:SetFrameStrata("HIGH")
    --RegisterStateDriver(au.AmendUIPetFollowStopFrame,"visibility","[combat][mod:ctrl]show;[nocombat,nomod:ctrl]hide")
    RegisterStateDriver(au.AmendUIPetFollowStopFrame,"visibility","[mod:ctrl]show;[nomod:ctrl]hide")


    PetActionButton2:SetParent(au.AmendUIPetFollowStopFrame)
    PetActionButton3:SetParent(au.AmendUIPetFollowStopFrame)

    PetActionButton2:ClearAllPoints()
    PetActionButton3:ClearAllPoints()
    PetActionButton2:SetPoint("BOTTOMLEFT",au.AmendUIPetFollowStopFrame)
    PetActionButton3:SetPoint("LEFT",PetActionButton2,"RIGHT",8,0)

    --Because of an oversight in the OnClick function for PetActionButtons, a ctrl-modified click
    --doesn't register properly on any of the buttons. Because secure functions can't be tampered with,
    --this is a hacky workaround to let the player click the Follow and Stay buttons while ctrl is
    --being held down.

    --Dummy "Follow" Button
    local pab2 = CreateFrame("Button", "AUIPetActionButton2", au.AmendUIPetFollowStopFrame, "SecureActionButtonTemplate");
    --Create a new invisible button right on top of the existing one
    pab2:SetAllPoints(PetActionButton2)
    pab2:SetFrameStrata("DIALOG")

    --Set this button to run a simple macro that will properly activate the desired command
    --This will also make it appear as if the button has been clicked
    pab2:SetAttribute("type", "macro")
    pab2:SetAttribute("macrotext", [=[/petfollow]=])
    pab2:RegisterForClicks("LeftButtonUp")

    --Dummy "Stay" Button
    local pab3 = CreateFrame("Button", "AUIPetActionButton3", au.AmendUIPetFollowStopFrame, "SecureActionButtonTemplate");
    --Create a new invisible button right on top of the existing one
    pab3:SetAllPoints(PetActionButton3)
    pab3:SetFrameStrata("DIALOG")

    --Set this button to run a simple macro that will properly activate the desired command
    --This will also make it appear as if the button has been clicked
    pab3:SetAttribute("type", "macro")
    pab3:SetAttribute("macrotext", [=[/petstay]=])
    pab3:RegisterForClicks("LeftButtonUp")

end


function au.PositionPetBar(reset)
    --If this setting is changed in combat lockdown, just ask the player to reload the UI
    if(reset and UnitAffectingCombat("player")) then
        au.ui_needs_reload = true
        return
    end
    if(reset) then
        for i=1,12 do
            local leftbutton = _G["MultiBarBottomLeftButton"..i]
            local rightbutton = _G["MultiBarBottomRightButton"..i]
            RegisterStateDriver(leftbutton,"visibility","show")
            RegisterStateDriver(rightbutton,"visibility","show")
            UnregisterStateDriver(leftbutton,"visibility")
            UnregisterStateDriver(rightbutton,"visibility")
        end
    end

    --1 "On Left Action Bar"
    --2 "Above Left Action Bar"
    --3 "On Right Action Bar"
    --4 "Above Right Action Bar"

    --string.upper(barside)
    local barside = "Left"
    local barbuttonstart = 3
    local bar_y = 0
    if(au.char_prefs.aui_petbarposition >= 3) then
        barside = "Right"
        barbuttonstart = 2
    end
    if(au.char_prefs.aui_petbarposition == 2 or au.char_prefs.aui_petbarposition == 4) then
        bar_y = 46
    else
        for i=barbuttonstart,barbuttonstart+6 do
            RegisterStateDriver(_G["MultiBarBottom" ..barside.. "Button"..i],"visibility","[@pet,noexists][@pet,dead]show;[@pet,nodead]hide")
        end
    end
    barbuttonstart = 44 + ((barbuttonstart-2) * 42)

    if(not au.PetBarFrame) then
        au.PetBarFrame = CreateFrame("Frame", "AmendUIPetBarFrame", UIParent, "SecureHandlerStateTemplate")
        RegisterStateDriver(au.PetBarFrame,"visibility","[@pet,noexists][@pet,dead]hide;[@pet,nodead]show")
    end
    --au.PetBarFrame:SetPoint("BOTTOMLEFT",MultiBarBottomRight,"BOTTOMLEFT",44,0);
    au.PetBarFrame:SetPoint("BOTTOMLEFT",_G["MultiBarBottom" ..barside],"BOTTOMLEFT",barbuttonstart,bar_y)

end

