local addonName, addonTable = ...
local au = AmendUIFuncs


function au.SetupHunterPetXPBar()
    -- ***** Pet XP Parent Frame *****
    au.PetXP = CreateFrame("Frame", "AmendUIPetXPFrame", PetFrameManaBar)
    au.PetXP:SetSize(104, 12)
    au.PetXP:ClearAllPoints()
    au.PetXP:SetPoint("BOTTOM", PetFrameManaBar, "BOTTOM", 12, -17)
    au.PetXP:SetFrameStrata("LOW")

    -- ***** Pet XP Status Bar *****
    au.PetXP.bar = CreateFrame("StatusBar", "AmendUIPetXPFrameStatusBar", au.PetXP)
    au.PetXP.bar:SetWidth(100)
    au.PetXP.bar:SetHeight(10)
    au.PetXP.bar:SetPoint("TOPLEFT", au.PetXP, "TOPLEFT", 2,-1)
    au.PetXP.bar:SetMinMaxValues(0, 100)
    au.PetXP.bar:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
    au.PetXP.bar:SetStatusBarColor(150/255, 100/255, 200/255)

    -- ***** Pet XP Border *****
    au.PetXP.bar.border = au.PetXP.bar:CreateTexture("AmendUIPetXPFrameStatusBarBorder", "OVERLAY")
    au.PetXP.bar.border:SetTexture("Interface\\Tooltips\\UI-StatusBar-Border")
    au.PetXP.bar.border:SetAllPoints(au.PetXP)

    -- ***** Pet XP BG *****
    au.PetXP.bar.bg = au.PetXP.bar:CreateTexture("AmendUIPetXPFrameStatusBarBG", "BACKGROUND")
    au.PetXP.bar.bg:SetTexture("Interface\\TARGETINGFRAME\\UI-StatusBar")
    au.PetXP.bar.bg:SetAllPoints(au.PetXP.bar)
    au.PetXP.bar.bg:SetVertexColor(0, 0, 0, 0.3)

    -- ***** Mouseover XP Text *****
    au.PetXP:SetScript('OnEnter', function()
        au.PetXP.bar.text:Show()
    end)
    au.PetXP:SetScript('OnLeave', function()
        au.PetXP.bar.text:Hide()
    end)

    au.PetXP.bar.text = au.PetXP.bar:CreateFontString("AmendUIPetXPFrameStatusBarText", "OVERLAY", "TextStatusBarText");
    au.PetXP.bar.text:SetPoint("CENTER", au.PetXP, "CENTER", 0 , 1);
    au.PetXP.bar.text:SetJustifyH("CENTER");
    au.PetXP.bar.text:SetJustifyV("CENTER");
    au.PetXP.bar.text:Hide()

    --barHidden means that the bar is always hidden (de-registers the Show/Hide conditions)
    au.PetXP.barHidden = false

    --barHooked keeps track of whether or not the par repositions scrips have been hooked.
    --Currently not used for anything.
    au.PetXP.barHooked = false

    --secureWaiting is set to 'true' if a function that does RegisterStateDriver is called during battle.
    --If 'true', the function will be called again when battle is over to prevent secure call errors
    au.PetXP.secureWaiting = false

    --Move Bar when buffs/debuffs are showing
    if(au.PetXP.barHooked == false) then
      au.PetXP.barHooked = true
      PetFrameBuff1:HookScript("OnShow",function() au.PetXP:SetPoint("BOTTOM", PetFrameManaBar, "BOTTOM", 13, -33) end)
      PetFrameBuff1:HookScript("OnHide",function() au.PetXP:SetPoint("BOTTOM", PetFrameManaBar, "BOTTOM", 13, -17) end)
    end

    au.PexXPBarEvents = {"UNIT_PET_EXPERIENCE", "UNIT_PET","PLAYER_REGEN_ENABLED"}

    for i, v in ipairs(au.PexXPBarEvents) do
      au.PetXP:RegisterEvent(v)
    end

    --Bar Updater
    au.PetXP:SetScript("OnEvent", function(self, event, ...)
  
        --Called when the pet earns experience
        if(event == "UNIT_PET_EXPERIENCE") then
          if(UnitLevel("player") ~= UnitLevel("pet")) then
            au.UpdatePetXPBar()
            if(au.PetXP.barHidden == true) then
              au.SetPetXPBarDisplayType()
            end
          else
            au.HidePetXPBar()
          end

        --Called when the pet is summoned or revived
        elseif(event == "UNIT_PET") then
          au.PetActiveCheck()

        --Called when combat lockdown ends
        elseif(event == "PLAYER_REGEN_ENABLED" and au.PetXP.secureWaiting == true) then
          if(au.PetXP.barHidden == true) then
            au.HidePetXPBar()
          else
            au.SetPetXPBarDisplayType()
          end
        end
    end)
    au.SetPetXPBarDisplayType()
end


--Checks to see if the player's pet is active. Updates and shows the XP bar if yes, hides it if no.
function au.PetActiveCheck()
  local hasUI, isHunterPet = HasPetUI();
  if (hasUI and isHunterPet) then
    if(UnitLevel("player") ~= UnitLevel("pet")) then
      au.UpdatePetXPBar()
      au.SetPetXPBarDisplayType()
    else
      au.HidePetXPBar()
    end
  end
end

--Updates the XP bar
function au.UpdatePetXPBar()
  currXP, nextXP = GetPetExperience()
  au.PetXP.bar:SetValue((currXP / nextXP) * 100)
  au.PetXP.bar.text:SetText(currXP .. " / " .. nextXP)
end

--Used to RegisterStateDriver for the XP bar based upon the player's saved preferences
function au.SetPetXPBarDisplayType(reset)
  if(UnitAffectingCombat("player")) then
    --Can't run RegisterStateDriver in combat; secureWaiting will signal that
    --this function needs to be called again after battle
    au.PetXP.secureWaiting = true
  else
    --1 - "Always"
    --2 - "Outside of Combat"
    if(au.char_prefs.aui_petxpdisplay == 2) then
      RegisterStateDriver(au.PetXP.bar,"visibility","[nocombat]show;[combat]hide")
    else--if(reset) then
      RegisterStateDriver(au.PetXP.bar,"visibility","show")
      UnregisterStateDriver(au.PetXP.bar,"visibility")
      au.PetXP:Show()
    end
    au.PetXP.secureWaiting = false
  end

  --Okay to set barHidden during combat even when secureWaiting gets flagged.
  --Whatever the most recent call was (show/false or hide/true) will be what
  --is used when the function is called again at the end of combat.
  au.PetXP.barHidden = false
end

function au.HidePetXPBar()
  if(au.PetXP.barHidden == true and au.PetXP.secureWaiting == false) then return end
  if(UnitAffectingCombat("player")) then
    --Can't run RegisterStateDriver in combat; secureWaiting will signal that
    --this function needs to be called again after battle
    au.PetXP.secureWaiting = true
  else
    RegisterStateDriver(au.PetXP.bar,"visibility","show")
    UnregisterStateDriver(au.PetXP.bar,"visibility")
    au.PetXP.secureWaiting = false
  end
  au.PetXP:Hide()

  --Okay to set barHidden during combat even when secureWaiting gets flagged.
  --Whatever the most recent call was (show/false or hide/true) will be what
  --is used when the function is called again at the end of combat.
  au.PetXP.barHidden = true
end
