local addonName, addonTable = ...
local au = AmendUIFuncs

function au.SetupMainMenuBar()
    if(au.global_prefs.Gryphons == true) then
        ReputationWatchBar:HookScript("OnShow",function() au.MMBGryphons() end)
        ReputationWatchBar:HookScript("OnHide",function() au.MMBGryphons() end)
        au.MMBGryphons()
    end
    if(au.global_prefs.MainMenuBar == true) then
        au.MMBLayout()
    end
    if(au.global_prefs.LandingButton == true) then
        au.MMBLandingButton()
    end
end

function au.MMBGryphons()
    if(au.global_prefs.Gryphons == false) then return end

    --The replacement texture for the end caps has to be a different size
    --when no xp or reputation bar is showing on the main HUD
    if(UnitLevel("player") == 60 and not ReputationWatchBar:IsShown()) then
        MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\AmendmentsUI\\Graphics\\UI-AmendEndCapLeftShorter.blp")
    else
        MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\AmendmentsUI\\Graphics\\UI-AmendEndCapLeft.blp")
    end
    MainMenuBarLeftEndCap:SetSize(32,64)
    MainMenuBarLeftEndCap:SetPoint("BOTTOM", -528, 0);

    MainMenuBarRightEndCap:SetTexCoord(0,1,0,1)
    if(UnitLevel("player") == 60 and not ReputationWatchBar:IsShown()) then
        MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\AmendmentsUI\\Graphics\\UI-AmendEndCapRightShorter.blp")
    else
        MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\AmendmentsUI\\Graphics\\UI-AmendEndCapRight.blp")
    end
    MainMenuBarRightEndCap:SetSize(32,64)
    MainMenuBarRightEndCap:SetPoint("BOTTOM", 528, 0);

end


function au.MMBLayout()
    if(au.global_prefs.MainMenuBar == false) then return end

    --Replacement textures for the MainMenuBar
    MainMenuBarTexture0:SetTexture("Interface\\AddOns\\AmendmentsUI\\Graphics\\UI-MainMenuBar-AmendUI.blp")
    MainMenuBarTexture1:SetTexture("Interface\\AddOns\\AmendmentsUI\\Graphics\\UI-MainMenuBar-AmendUI.blp")
    MainMenuBarTexture2:SetTexture("Interface\\AddOns\\AmendmentsUI\\Graphics\\UI-MainMenuBar-AmendUI.blp")
    MainMenuBarTexture2:SetTexCoord(0, 1, 0.33203125, 0.5);
    MainMenuBarTexture3:SetTexture("Interface\\AddOns\\AmendmentsUI\\Graphics\\UI-MainMenuBar-AmendUI.blp")
    MainMenuBarTexture3:SetTexCoord(0, 1, 0.08203125, 0.25);

    --Deal with the KeyRing button. Only show to the side of the main HUD
    --when an inventory bag is open
    MainMenuBar_UpdateKeyRing = function() end
    KeyRingButton:SetParent(ContainerFrame1)
    KeyRingButton:ClearAllPoints()
    KeyRingButton:SetPoint("LEFT", MainMenuBarBackpackButton,"RIGHT",10,0)
    KeyRingButton:Show()

    --Create the seven new action buttons that will show on the tweaked MainMenuBar
    au.ActionBarPosition = CreateFrame( "Frame","AmendUIActionBarPosition",MainMenuBarArtFrame)
    au.ActionBarPosition:SetSize(36,36)
    au.ActionBarPosition:SetPoint("BOTTOMLEFT", _G["ActionButton12"],"BOTTOMRIGHT", 54, 18)

    --Only 120 ActionButtons exist in the default UI - best to use the IDs for these buttons
    --to ensure that they work just like the defaults. ActionButtons 109 through 120 aren't
    --really used by any class or extra bar, so they're safe to use for this purpose
    for id=109,115 do
        local bnum = id-108
        local exbb = CreateFrame("CheckButton", "AmendUIBarButton"..bnum, MainMenuBarArtFrame, "ActionBarButtonTemplate")
        exbb:SetID(0)
        exbb:SetAttribute("action", id)
        exbb:SetPoint("LEFT", _G["AmendUIActionBarPosition"], "BOTTOMLEFT", (42*(id-109)), 0)
        exbb:SetFrameStrata("MEDIUM")
        exbb:Show()
        _G["AMENDUIBARBUTTON" ..bnum] = exbb
        --print("ButtonCreated: " ..exbb:GetName())
    end

    CharacterBag0Slot:SetScale(0.50)
    CharacterBag0Slot:ClearAllPoints()
    CharacterBag0Slot:SetPoint("RIGHT", "MainMenuBarBackpackButton", "LEFT", -18, -13)
    for i=1,3 do
        _G["CharacterBag"..i.."Slot"]:SetScale(0.50)
        _G["CharacterBag"..i.."Slot"]:ClearAllPoints()
        _G["CharacterBag"..i.."Slot"]:SetPoint("RIGHT", "CharacterBag"..(i-1).."Slot", "LEFT", -13, 0)
    end

    --This popout menu button will show and hide the MicroMenuButtons
    local MPPButton = CreateFrame("Button", "MicroPopoutPanelButton", MainMenuBarArtFrame)

    MPPButton:SetNormalTexture("Interface\\AddOns\\AmendmentsUI\\Graphics\\UI-AmendMicroPopoutButton-Up.blp")
    MPPButton:SetPushedTexture("Interface\\AddOns\\AmendmentsUI\\Graphics\\UI-AmendMicroPopoutButton-Down.blp")
    MPPButton:SetHighlightTexture("Interface\\AddOns\\AmendmentsUI\\Graphics\\UI-AmendMicroPopoutButton-Disabled.blp")

    MPPButton:SetSize(16, 32)
    MPPButton:SetPoint("RIGHT", "CharacterBag3Slot", "LEFT", -6, 8)
    MPPButton:SetFrameStrata("HIGH")

    MPPButton:SetScript("OnClick", function()
        if _G["MicroPopoutPanelBox"]:IsVisible() then
            _G["MicroPopoutPanelBox"]:Hide()
            _G["MicroPopoutPanelButton"]:SetNormalTexture("Interface\\AddOns\\AmendmentsUI\\Graphics\\UI-AmendMicroPopoutButton-Up.blp")
            _G["MicroPopoutPanelButton"]:SetPushedTexture("Interface\\AddOns\\AmendmentsUI\\Graphics\\UI-AmendMicroPopoutButton-Down.blp")
            _G["MicroPopoutPanelButton"]:SetHighlightTexture("Interface\\AddOns\\AmendmentsUI\\Graphics\\UI-AmendMicroPopoutButton-Disabled.blp")
        else
            _G["MicroPopoutPanelBox"]:Show()
            _G["MicroPopoutPanelButton"]:SetNormalTexture("Interface\\AddOns\\AmendmentsUI\\Graphics\\UI-AmendMicroPopoutButtonReverse-Up.blp")
            _G["MicroPopoutPanelButton"]:SetPushedTexture("Interface\\AddOns\\AmendmentsUI\\Graphics\\UI-AmendMicroPopoutButtonReverse-Down.blp")
            _G["MicroPopoutPanelButton"]:SetHighlightTexture("Interface\\AddOns\\AmendmentsUI\\Graphics\\UI-AmendMicroPopoutButtonReverse-Disabled.blp")

        end
    end)

    local AmendUIMicroPopoutBox = CreateFrame("Frame", "MicroPopoutPanelBox", UIParent)
    AmendUIMicroPopoutBox:SetFrameStrata("HIGH")
    AmendUIMicroPopoutBox:SetPoint("BOTTOMLEFT", "MicroPopoutPanelButton", "BOTTOMRIGHT", 0, -4)

    AmendUIMicroPopoutBox.texture = AmendUIMicroPopoutBox:CreateTexture()
    AmendUIMicroPopoutBox.texture:SetAllPoints(AmendUIMicroPopoutBox)
    AmendUIMicroPopoutBox.texture:SetColorTexture(0,0,0,0.7)
    AmendUIMicroPopoutBox:Hide()

    --Setup micro menu popout based upon whether or not the talent button is being shown
    au.RefreshMicroPanelPopoutBox(UnitLevel("player"))

    --This will check for a level up event to make sure the micro menu popout menu
    --gets properly reconfigured when the player goes from level 9 to 10.
    if(UnitLevel("player") < 10) then
      au.Level10Check = CreateFrame("Frame")
      au.Level10Check:RegisterEvent('PLAYER_LEVEL_UP')
      au.Level10Check:SetScript('OnEvent', function(_, _, level)
      	if(level == 10) then
          au.RefreshMicroPanelPopoutBox(10)
        end
      end)
    end

    --MainMenuBarPerformanceBarFrame:Hide()
    MainMenuBarPerformanceBarFrame:SetParent(AmendUIMicroPopoutBox)
    MainMenuBarPerformanceBarFrame:ClearAllPoints()
    MainMenuBarPerformanceBarFrame:SetPoint("TOPLEFT","MicroPopoutPanelBox","TOPRIGHT",-14,14)
    MainMenuBarPerformanceBarFrame:SetFrameStrata("HIGH")

    --Prevent default UI functions from messing with the micro buttons
    --This happenened most noticably when starting a new character
    UpdateMicroButtonsParent = function() end
    MoveMicroButtons = function() end
end

--Before level 10 the Talent micro button isn't shown.
--This function sets up the micro popout menu and
--refreshes it when a player levels up from 9 to 10
function au.RefreshMicroPanelPopoutBox(plevel)
  if(plevel >= 10) then
    _G["MicroPopoutPanelBox"]:SetSize(232, 40)
  else
    _G["MicroPopoutPanelBox"]:SetSize(208, 40)
  end

  CharacterMicroButton:ClearAllPoints()
  CharacterMicroButton:SetPoint("BOTTOMLEFT", "MicroPopoutPanelButton", "BOTTOMRIGHT", 2, -3)

  for _, v in ipairs(MICRO_BUTTONS) do
      v = _G[v]
      v:SetParent(_G["MicroPopoutPanelBox"])
  end
end


-- ********** DISPLAY GOLD HUD **********
function au.DisplayGold_OnEvent()
    local copper, silver, gold
    copper = GetMoney()
    gold = floor(copper / 10000)
    silver = floor((copper / 100) % 100)
    copper = copper % 100

    local moneyxpos = -180

    if gold < 1 then
        au.gdisplay:SetText("|cFFEEEEEE"..silver.."|r|TInterface\\MONEYFRAME\\UI-SilverIcon:0:0:2|t|cFFFFCFA3 "..copper.."|TInterface\\MONEYFRAME\\UI-CopperIcon:0:0:2|t")
    elseif gold < 1000 then
        moneyxpos = -176
        au.gdisplay:SetText("|cFFFFFF99"..gold.."|r|TInterface\\MONEYFRAME\\UI-GoldIcon:0:0:2|t|cFFEEEEEE "..silver.."|r|TInterface\\MONEYFRAME\\UI-SilverIcon:0:0:2|t|cFFFFCFA3 "..copper.."|TInterface\\MONEYFRAME\\UI-CopperIcon:0:0:2|t")
    else
        au.gdisplay:SetText("|cFFFFFF99"..gold.."|r|TInterface\\MONEYFRAME\\UI-GoldIcon:0:0:2|t|cFFEEEEEE "..silver.."|r|TInterface\\MONEYFRAME\\UI-SilverIcon:0:0:2|t")
    end

    au.MMBGoldFrame:SetPoint("LEFT",_G["MainMenuBarArtFrame"],"RIGHT",moneyxpos,0)
    --MMBGoldFrame:SetPoint("BOTTOM",UIParent,"BOTTOM",0,23)
end

function au.SetupGoldHUD()
    if(au.global_prefs.MainMenuBar == false) then return end
    au.MMBGoldFrame = CreateFrame("Frame", "MMBGoldFrame", MainMenuBarArtFrame)
    au.MMBGoldFrame:SetPoint("BOTTOM",UIParent,"BOTTOM",0,25)
    au.MMBGoldFrame:SetSize(120, 15)
    au.gdisplay = MMBGoldFrame:CreateFontString("GoldDisplay","BACKGROUND")
    au.gdisplay:SetFontObject(NumberFontNormal)
    au.gdisplay:SetFont("Fonts\\ARIALN.ttf",13)
    au.gdisplay:SetSize(120, 15)
    au.gdisplay:SetJustifyH("RIGHT")
    au.gdisplay:SetJustifyV("CENTER")
    au.gdisplay:SetPoint("LEFT")

    au.MMBGoldFrame:SetScript("OnEvent", au.DisplayGold_OnEvent)
    au.MMBGoldFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
    au.MMBGoldFrame:RegisterEvent("PLAYER_MONEY")
end

-- **************************************

--This implementation isn't ideal, but this button is tied into a ton of Blizzard UI
--repositioning functions. It may become more important in later expansions.
function au.MMBLandingButton()
    if(au.global_prefs.MMBLandingButton == false) then return end
    MainMenuBarVehicleLeaveButton:ClearAllPoints()
    MainMenuBarVehicleLeaveButton:SetPoint("RIGHT", MainMenuBarLeftEndCap, "LEFT", 10, -5)
    MainMenuBarVehicleLeaveButton.ClearAllPoints = function() end
    MainMenuBarVehicleLeaveButton.SetPoint = function() end
end




